/* -*-C++-*- */
/*
 *  Copyright (c) 1999 Regents of the University of California.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley
 *       National Laboratory.
 *  4. Neither the name of the University nor of the Laboratory may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 */

/* -*-C++-*- */
#ifndef _SIMPLE_ACL_AUTHORIZER_
#define _SIMPLE_ACL_AUTHORIZER_

//---------------------------------------------------------------------
// A concrete extension of Authorizer that uses a simple acl file that 
// contains only a newline seperated list of names.  There are no actions
// associated with the name.  A call to authorize will return true if and 
// only if the name passed in is in the acl file.
//-----------------------------------------------------------------------

#include <string>
#include <map>
#include "Authorizer.h"

typedef std::map <std::string, std::string, std::less<std::string> >
strStrMap;

class SimpleACLAuthorizer : public Authorizer
{
  public:
//---------------------------------------------------------------
// The constructors take a single argument.  It is a string containing
// the full path to the acl file.  A string will be thrown if unable to
// open the file, or if the file is corrupted in some way.
//-----------------------------------------------------------------
    SimpleACLAuthorizer(const std::string& file);
    SimpleACLAuthorizer(const char* file);
    virtual ~SimpleACLAuthorizer( void );
//-------------------------------------------------------------------
// These authorize methods take a principal name, and return true if 
// the name is in the acl file, and false in all other cases.
//-------------------------------------------------------------------
    virtual bool authorize(const std::string& name,
			   const std::string& rqstAct = "");
    virtual bool authorize(const char* name, const char* rqstAct = "");
//---------------------------------------------------------------------
// The following authorize method is unimplimented in this class,
// since there are no actions associated with a name.
//---------------------------------------------------------------------
    virtual void authorize(const std::string& name, std::string* allwdActs);
  protected:
    virtual bool checkAction(std::string& rqstAct, std::string& actList);
    strStrMap aclMap;
  private:
    void readACLFile(const char* file);
    void parseLine(std::string& line);
};

#endif // _SIMPLE_ACL_AUTHORIZER_


