/* -*-C++-*- */
/*
 *  Copyright (c) 1999 Regents of the University of California.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley
 *       National Laboratory.
 *  4. Neither the name of the University nor of the Laboratory may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 */


/* -*-C++-*- */
/*
 *  This TestInfo class handles the variables
 *  that are common to all network tests. 
 */
#ifndef TESTINFO_H
#define TESTINFO_H

#include <iosfwd>
#include <string>
//#include "TestInfo.h"
#include "MyDebug.h"

class Results;

using std::string;
using std::ostream;

enum TerminationTypes {Timed, NumBytes, NumMsgs, NoTerm};

class TestInfo : public MyDebug
{
  public:
    TestInfo();
    TestInfo(bool val, const string& address, const string& interface,
	     int portVal, int bufLenVal, double reportInterval, 
	     TerminationTypes termVal, double testAmount);
    TestInfo( const TestInfo& tInfo );

    virtual ~TestInfo(void);

    inline void setResults (Results* r_ptr)
    {
	rContainer = r_ptr;
    };
    inline void setIsSender( bool isSender )
    {
	mIsSender = isSender;
    };
    inline void setLocalTestInterface( const string& address )
    {
	mLocalTestInterface = address;
    };
    inline void setDelay( int delay )
    {
	delay_secs = delay;
    };
    inline void setPort( int port )
    {
	mPort = port;
    };
    inline void setBufferLen( int bufLen )
    {
	mBufferLen = bufLen;
    };
    inline void setReportInterval( double interval )
    {
	mReportInterval = interval;
    };
    inline void setTermination( TerminationTypes mode )
    {
	mTermination = mode;
    };
    inline void setTestAmount( double amt )
    {
	mTestAmount = amt;
    };
    inline void setLocalName( const string& host )
    {
	mLocalName = host;
    };
    inline void setRemoteTestPeer( const string& host )
    {
	mRemoteTestPeer = host;
    };
    inline void setTestParams( const string& params )
    {
        mTest_Params = params;
    };
    inline  Results* getResults( void )
    {
	return rContainer;
    };
    inline const string& getLocalName( void ) const
    {
	return mLocalName;
    };
    inline const string& getRemoteTestPeer( void ) const
    {
	return mRemoteTestPeer;
    };
    inline const string&  getLocalTestInterface( void ) const 
    {
	return mLocalTestInterface;
    };
    inline const string getTestParams() const
    {
        return mTest_Params;
    };
    inline int getDelay( void ) const
    {
	return delay_secs;
    };
    inline int getPort( void ) const 
    {
	return mPort;
    };
    inline int getBufferLen( void ) const 
    {
	return mBufferLen;
    };
    inline double getReportInterval( void ) const
    {
	return mReportInterval;
    };
    inline TerminationTypes getTermination( void ) const
    {
	return mTermination;
    };
    inline double getTestAmount( void ) const
    {
	return mTestAmount;
    };
    inline bool isSender( void ) const 
    {
	return mIsSender;
    };
    virtual ostream &  printVariables(ostream &);
    virtual TestInfo& operator=( const TestInfo& tInfo );
    virtual void setUserParams( void );

  protected:
    bool mBufLenSet;
    bool mIsSender; // boolean indication of whether this is the (initial) sender

    string mLocalName;
    string mRemoteTestPeer;      // address/name of remote test host
    string mLocalTestInterface;  // which local interface to bind to for the test
    string mSender;
    string mReceiver;
    string mTest_Params;

    int              delay_secs;      // seconds to wait before test begins
    int              mPort;           // port to use for test
    int              mBufferLen;      // size of each message sent
    double           mReportInterval; // frequency for reporting test results
    TerminationTypes mTermination;    // termination mode
    double           mTestAmount;     // # of bytes, # of messages, or time for test to run
    int              mDebugLevel;     // how much debugging info to print out during test
    Results*         rContainer;

    void getToken( string& ans, const string& params, const string& symbols,
                         const string::size_type& begIdx );
};

#endif /*  TESTINFO_H */
