/* -*-C++-*- */
/* Copyright (c) 1999 Regents of the University of California.
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley 
 *       National Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: network.h,v 1.7 2001/09/12 21:13:03 dang Exp $
*/


// network.h: Base class for network -- multi-threaded version. 

#ifndef Network_h
#define Network_h

#include <string>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <sys/types.h>
#include <cassert>
#include <ctype.h>      // for isdigit() with CC compiler
#ifndef WIN32 
#include <sys/socket.h>
#include <pthread.h>

#ifdef sun
#include <sys/systeminfo.h> 
#endif

#ifdef linux
#include <sys/utsname.h>
#endif

#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sysexits.h>
#include <cerrno>
#include <unistd.h>	// for close() with CC compiler
#include <sys/time.h>
#endif

#include "MyDebug.h"

using std::string;

#define CLASSD_ADDR(x)  (((u_long)(x) & ((u_long)0xf0000000)) == \
			     ((u_long)0xe0000000))

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN  256
#endif

class Network : public MyDebug {
public:
	Network(int=0,int=0,short=5555);
        virtual ~Network();
	void setPort(short val );  // set the socket port
	void setBlockTime( int secs, int usecs );  // set the doRecv blocking time
	inline short getPort() const { return port; }  // returns the current port
	string* getLocalName(void);  // gets the hostname of the local machine
	string* getRemoteHost(void);  // gets the hostname of the remote machine
	//virtual int openSocket() = 0;  // opens the socket
	virtual int doSend(const char *, long);  // sends a message on the socket
	virtual int doRecv(char *, long);  // receives the next message (blocks if block_* variables set)
	virtual int doBlockedRecv(char *, long, char *); // receive the next message (block until received)
	inline int getTimeout() { return block_secs; }  // return the number of seconds a doRecv will block
	static int isClassDAddr(const char *);  // check if an address is multicast
	inline int getNetSock() { return sock; }

protected:
	int sock;                      // socket identifier
	int block_secs, block_usecs;   // time to block on a call to doRecv
	short port;                    // socket port
	struct sockaddr_in peer;       // struct that defines socket parameters
	// mutexes for locking receive/send buffers
	static pthread_mutex_t rmutex, smutex;  // receive and send mutexes
	static unsigned long convertAddr(const char *s);  // convert to 32 bit inetaddr
};

#endif
