/* -*-C-*- */
/*
 *  Copyright (c) 1999 Regents of the University of California.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley
 *       National Laboratory.
 *  4. Neither the name of the University nor of the Laboratory may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 *  $Id: timefuncs.c,v 1.3 2001/08/22 16:34:35 boverhof Exp $
 */


/**************************************
**      timefuncs.c
**
**    This file contains all of the routines
**    Needed for all of the time calculations
**    So that times can be printed, compared,
**    converted to reals and any of the other
**    things needed.
******************************************/

#ifdef WIN32
#include <sys/timeb.h>
#include		<winsock2.h>
#else
#include <sys/time.h>
#endif //WIN32
#include <sys/types.h>
#include <netinet/in.h>
/*#include <inttypes.h>*/

#ifdef WIN32
void timebtotimev( struct _timeb *t1, struct timeval *t2)
{
	/* copies the appropriate fields of a windows _timeb
	*  struct to a timeval struct */
	t2->tv_sec = (long)t1->time;
	t2->tv_usec = (long)(t1->millitm) * 1000; 
}
#endif //WIN32

long comparetimev( struct timeval *t1, struct timeval *t2 )
{
	/* compares the two times and returns a positive
	** number if t1 is larger and a negative number
	** if t2 is larger.  Zero is returned if they are
	** equal.
	*/
	long temp;
	temp = t1->tv_sec - t2->tv_sec;
	if( temp != 0 )
		return temp;
	temp = t1->tv_usec - t2->tv_usec;
	return temp;
}

double timevtodouble( struct timeval *t1 )
{
	/*  convert a timeval into a double and return it */
	double temp;
	temp = (double)(t1->tv_sec);
	temp += ((double)(t1->tv_usec))/1000000.0;
	return temp;
}

double timevtomsec( struct timeval *t1 )
{
    return (((double)(t1->tv_sec)*1000.0) + 
                   ((double)(t1->tv_usec)/1000.0));
}

struct timeval subtimev( struct timeval *t1, struct timeval *t2)
{
        /* subtract time t2 from time t1 */
	struct timeval temp;

	temp.tv_sec = t1->tv_sec - t2->tv_sec;
	temp.tv_usec = t1->tv_usec - t2->tv_usec;
		if( temp.tv_usec < 0 )
		{
			temp.tv_usec += 1000000;
			temp.tv_sec -= 1;
		}
	return temp;
}

struct timeval absdifftimev( struct timeval *t1, struct timeval *t2)
{
	/* find the absolute difference between time t1 and t2 
	** and return the result as a timeval */
	struct timeval temp;
	long cmpr;

	cmpr = comparetimev( t1, t2 );
	if( cmpr == 0 )
	{
		temp.tv_sec = 0;
		temp.tv_usec = 0;
		return temp;
	}
	if( cmpr > 0 )
		return subtimev( t1, t2 );
	else
		return subtimev( t2, t1 ); 
}

void ntohltimev( struct timeval *t1)
{
	t1->tv_sec = ntohl(t1->tv_sec);
	t1->tv_usec = ntohl(t1->tv_usec);
}

void addtimev( struct timeval *t1, struct timeval *t2)
{ /* add the time t2 to the time t1 and place result in t1 */
	t1->tv_sec += t2->tv_sec;
	t1->tv_usec += t2->tv_usec;
	if( t1->tv_usec >= 1000000 )
	{
		t1->tv_usec -= 1000000;
		t1->tv_sec += 1;
	}
	return;
}
