/* -*-C++-*- */
/* Copyright (c) 1999 Regents of the University of California.
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley
 *       National Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <inttypes.h>

#include "Display_funcs.h"
#include "Results.h"
#include "CoordInfo.h"
#include "network.h"
#include "config.h"
#include "funcs.h"

using std::cout;
using std::endl;

DisplayFunc::DisplayFunc()
{
   abbr[0] = 'u';
   abbr[1] = 'm';
   abbr[2] = 'i';
   abbr[3] = 't';
   abbr[4] = 'c';
   abbr[5] = 'o';
   abbr[6] = 'w';
   abbr[7] = 'p';
   abbr[8] = 'D';
   abbr[10] = 'q';

   tests_Map.insert( make_pair('u', static_cast<string>("udpping")) );
   tests_Map.insert( make_pair('m', static_cast<string>("multiping")) );
   tests_Map.insert( make_pair('i', static_cast<string>("iperf")) );
   tests_Map.insert( make_pair('t', static_cast<string>("tuned")) );
   tests_Map.insert( make_pair('c', static_cast<string>("master")) );
   tests_Map.insert( make_pair('o', static_cast<string>("reports")) );
   tests_Map.insert( make_pair('w', static_cast<string>("traceroute")) );

   tests_Map.insert( make_pair('p', static_cast<string>("pipechar")) );
   tests_Map.insert( make_pair('D', static_cast<string>("debug")) );

   tests_Map.insert( make_pair('q', static_cast<string>("quit")) );
}

DisplayFunc::~DisplayFunc()
{
}

/*----------------------------------------------------------------------
 *  displayUsage:
 *
 *
 *----------------------------------------------------------------------*/

void 
DisplayFunc::displayUseage()
{
    cout << "Bad argument supplied" << endl;
    cout << "Arguments:" << endl;
    cout << "\t [-h hostname] - Declare the test Master" << endl;
    cout << "\t [-c filename] - client-side pem certificate filename (default nettest.pem)" << endl;
    cout << "\t [-k filename] - client-side pem key filename (default key.pem)" << endl;
#if USE_NETLOGGER
    cout << "\t [-n hostname] - Send NetLogger messages to netlogd on hostname" << endl;
#endif
    cout << "\t [-a passphrase] - Passphrase for client-side if NOT default (key.pem, nettest.pem)" << endl;
    cout << "\t [-b passphrase] - Passphrase to decrypt nettest.pem's private key key.pem " << endl;
    cout << "\t [-s] - this is a server who waits for a connection" << endl;
    cout << "\t [-d] - Run As Daemon process" << endl;
    cout << "\t [-D number] - Debugging level" << endl;
}


/*----------------------------------------------------------------------
 *  displayAskUser
 *	string& ans
 *
 *
 *----------------------------------------------------------------------*/

string
DisplayFunc::displayAskUser( )
{
    cout << "\nSelect an option\n";
    cout << "\t" << abbr[0] << " = udp connectivity and round trip time\n";
    cout << "\t" << abbr[1] << " = multicast connectivity and round trip time\n";
    cout << "\t" << abbr[2] << " = tcp throughput (iperf test)\n";
#ifdef PATH_TRACEROUTE
    cout << "\t" << abbr[6] << " = traceroute\n"; 
    cout << "\t" << abbr[7] << " = pipechar\n"; 
#endif
    cout << "\t" << abbr[3] << " = tuned tcp (determine optimal tcp parameters)\n";
    cout << "\t" << abbr[4] << " = change Test Master\n";
#if USE_NETLOGGER
    cout << "\t" << abbr[5] << " = change Requester's result reporting(printing, Netlogging)\n"; 
#else
    cout << "\t" << abbr[5] << " = change Requester's result reporting(printing to stdout)\n";
#endif
    cout << "\t" << abbr[8]  << " = change Debug Level\n";
    cout << "\t" << abbr[10] << " = quit\n"; 

    string ans;
    getAnswer(ans);
    if(!ans.empty()){
        char t = ans[0];
        pos = tests_Map.find(t);
        if( pos == tests_Map.end() ) return "";
        return pos->second;
    }

    return "";
}


/*----------------------------------------------------------------------
 *  displayChangedReportResults 
 *----------------------------------------------------------------------*/

void
DisplayFunc::displayChangedReportResults( Results* rptr )
{
    if( rptr->getIsNLenabled() )
    {
    	cout << "\n****************************************";
    	cout << "\n  Iperf      print     output:  " << ( rptr->getIsPrintIperf() )? "YES": "NO";
        cout << "\n             Netlogger output:  " << ( rptr->getIsNLiperf() )? "YES": "NO";
    	cout << "\n  Ping       print     output:  " << ( rptr->getIsPrintPing() )? "YES": "NO";
        cout << "\n             Netlogger output:  " << ( rptr->getIsNLping() )? "YES": "NO";
#ifdef PATH_TRACEROUTE
    	cout << "\n  traceroute print     output:  " << ( rptr->getIsPrintSO() )? "YES": "NO";
        cout << "\n             Netlogger output:  " << ( rptr->getIsNLSO() )? "YES": "NO";
#endif
    	cout << "\n****************************************" << endl << endl;
    }
    else
    {
    	cout << "\n****************************************";
    	cout << "\n  Iperf      print     output:  " << ( rptr->getIsPrintIperf() )? "YES": "NO";
    	cout << "\n  Ping       print     output:  " << ( rptr->getIsPrintPing() )? "YES": "NO";
#ifdef PATH_TRACEROUTE
    	cout << "\n  traceroute print     output:  " << ( rptr->getIsPrintSO() )? "YES": "NO";
#endif
    	cout << "\n****************************************" << endl << endl;
    }

    if( rptr->getIsNLenabled() )
    {
        if (askQuestion("Do you want to change result reporting? [no] ")>0){
            rptr->setIsNLiperf( (askQuestion("\tDo you want iperf NetLogger ULM output? [no] ")>0) );
            rptr->setIsPrintIperf( (askQuestion("\tDo you want iperf output printed to the screen? [no] ")>0) );
            rptr->setIsNLping( (askQuestion("\tDo you want ping NetLogger ULM output? [no] ")>0) );
            rptr->setIsPrintPing( (askQuestion("\tDo you want ping output printed to the screen? [no] ")>0) );
            rptr->setIsNLSO( (askQuestion("\tDo you want traceroute NetLogger ULM output? [no] ")>0) );
            rptr->setIsPrintSO( (askQuestion("\tDo you want traceroute output printed to the screen? [no] ")>0) );
        }
    }
    else
    {
        if (askQuestion("Do you want to change result reporting? [no] ")>0){
            rptr->setIsPrintIperf( (askQuestion("\tDo you want iperf output printed to the screen? [no] ")>0) );
            rptr->setIsPrintPing( (askQuestion("\tDo you want ping output printed to the screen? [no] ")>0) );
            rptr->setIsPrintSO( (askQuestion("\tDo you want traceroute output printed to the screen? [no] ")>0) );
        }
    }
}


/*----------------------------------------------------------------------
 *  displayGetAcceptHost
 *
 *----------------------------------------------------------------------*/

void    
DisplayFunc::displayGetAcceptHost(  const string masterName, string& ahost )
{
    string contin;

    cout << "\n-----------------------------------------------------------\n";
    cout << "   Select the Host which will accept the new connection\n";
    cout << "\t Enter the host's name or type return [] to choose " << masterName << endl;
    cout << "\n-----------------------------------------------------------\n";

    cout << "\nHost to Accept: ";
    getFirstWord(ahost);
}


/* -------------------------------------------------------------------
 *  displayGetTestHosts
 *
 * ------------------------------------------------------------------- */

void 
DisplayFunc::displayGetTestHosts()
{
    cout << "\n-------------------------------------------------------------\n";
    cout << "   Enter test Receiver and Sender Names\n";
    cout << "   or Return[] for Default \n";
    cout << "\n-------------------------------------------------------------\n";
}


/* -------------------------------------------------------------------
 *  getUseDefaults
 *
 * ------------------------------------------------------------------- */

int 
DisplayFunc::getUseDefaults( const string testSender, const string testReceiver )
{
    cout << "\n-------------------------------------------------------------\n";
    cout << "  Sender Receiver: Use default test parameters/repeat last test.";
    cout << "\n\tSender: " << testSender << "\n\tReceiver: " << testReceiver;
    cout << "\n-------------------------------------------------------------\n";
    cout << "\nTo specify test parameters[y] or return for default[]: ";

    return isContinue();
}


int 
DisplayFunc::getUseDefaults_SO( const string testSender, const string testReceiver )
{
    cout << "\n-------------------------------------------------------------\n";
    cout << "  Sender Only: Use default test parameters/repeat last test.";
    cout << "\n\tSender: " << testSender << "\n\tTarget: " << testReceiver;
    cout << "\n-------------------------------------------------------------\n";
    cout << "\nTo specify test parameters[y] or return for default[]: ";

    return isContinue();
}

/* -------------------------------------------------------------------
 *  getMaster
 *
 * ------------------------------------------------------------------- */

void 
DisplayFunc::getMaster( string& master )
{
    cout << "\n-------------------------------------------------------------\n";
    cout << "  Enter new Master or Return[] for " << master;
    cout << "\n-------------------------------------------------------------\n";
    cout << "\nMaster:  " ;

    getFirstWord( master );
}


/* ---------------------------------------------------------------------------------
 * getTestHosts: Function to get Test Site names.  If Test Sites are already specified
 *	in CoordInfo, this function does not check them.
 *
 * --------------------------------------------------------------------------------- */

void 
DisplayFunc::getTestHosts( CoordInfo& c_info )
{
    displayGetTestHosts();
    string testSite;

    while( 1 )
    {
        do{
	    cout << endl << "Who is the Sender? [" << c_info.getTestSender() << "]  ";
            getFirstWord( testSite );
	    if( testSite.empty() && !(c_info.getTestSender()).empty() )
	        break;
        } while( testSite.empty() || getCanonicalName( testSite )==-1 );

        if( !testSite.empty() )
	    c_info.setTestSender( testSite );

        do{
	    ( c_info.getIsSOTest() )? 
		cout << endl << "Who is the Target?   [" << c_info.getTestReceiver() << "]  ":
            	cout << endl << "Who is the Receiver? [" << c_info.getTestReceiver() << "]  ";

            getFirstWord( testSite );
	    if( testSite.empty() && !(c_info.getTestReceiver()).empty() )
		    break;
        } while( testSite.empty() || getCanonicalName( testSite )==-1 );

        if( !testSite.empty() ) 
	    c_info.setTestReceiver( testSite );

        cout << "\n\tSender  : " <<  c_info.getTestSender() << endl;
        cout << "\tReceiver: "   <<  c_info.getTestReceiver() << endl;
/*
        if( c_info.getTestSender() == c_info.getTestReceiver()  )
            cout << "Sender and Receiver cannot be the same\n";
        else
*/
	break;
    }
}


/* -------------------------------------------------------------------
 *  displayAskUserIperf
 *
 * ------------------------------------------------------------------- */

void    
DisplayFunc::changeDebugLevel( string& debugL )
{
    cout << "\n****************************************";
    cout << "\n Current DebugLevel: " << debugL;
    cout << "\n  off - 0   max - 5"; 
    cout << "\n****************************************\n\n";

    string temp;

    cout << "Enter Debug Level = ";
    getFirstWord( temp );

    if( !temp.empty() && (temp[0]>='0' && temp[0]<'6') )
	debugL = temp[0];
    else
	cout << "\nERROR: DebugLevel not between 0 to 5\n";
	
    return;
}


/* -------------------------------------------------------------------
 *  displayAskUserIperf
 *
 * ------------------------------------------------------------------- */

void
DisplayFunc::displayAskUserIperf()
{
    cout << "\nChoose Iperf test parameters, begin each with - (eg. -P 2 )\n";
    cout << "\td [seconds]   - Delay start of test\n";
    cout << "\tf [character] - format output K for kilobits, or M for megabits\n";
    cout << "\ti [seconds]   - interval between periodic reports\n";
    cout << "\tl [bytes]     - buffer size\n";
    cout << "\tn [bytes]     - termination bytes of data\n";
    cout << "\tp [number]    - port\n";
    cout << "\tt [seconds]   - termination in test duration seconds\n";
    cout << "\tP [number]    - number of parallel running clients\n\n";
    cout << "\tu             - UDP Datagram iperf test\n";
    cout << "\tb [number]    - UDP bandwidth to send atin bits/sec, must use -u\n";
    cout << "\tw [bytes]     - TCP window size\n";
    cout << "\tM [bytes]     - TCP maximum segment size\n";
    cout << "\tm [bytes]     - print reported TCP MSS size\n";
    cout << "\tN             - TCP no delay option\n\n";
    cout << "\te             - cumulative stats at the end-of-test\n";

    //cout << "\tT [number]    - time-to-live(hops) for multicast\n";

    //UNIMPLEMENTED FEATURES BELOW
    //cout << "\tS             - type-of-service for outgoing packets\n";
}


/* -------------------------------------------------------------------
 *  displayResult
 *
 * ------------------------------------------------------------------- */

void
DisplayFunc::displayResult( const string message, const string answer )
{
    cout << message << answer << endl;
}


/* -------------------------------------------------------------------
 *  displayAcceptHost
 *
 * ------------------------------------------------------------------- */

void 
DisplayFunc::displayAcceptHost( const string ahost )
{
    cout << "\n****************************************";
    cout << "\n Accept Host: " << ahost;
    cout << "\n****************************************\n\n";
}


/* -------------------------------------------------------------------
 *  displayCurrentTest
 *
 * ------------------------------------------------------------------- */

void    
DisplayFunc::displayCurrentTest( const string params )
{
    cout << "\n****************************************";
    cout << "\n    *** Sender-Recv TEST  REQUEST ***\n";
    cout << "\n Debug / Test /  master  /  sender  /   receiver   /   parameters\n";
    cout << endl << params << endl;
    cout << "\n****************************************\n\n";
}

void    
DisplayFunc::displayCurrentTest_SO( const string params )
{
    cout << "\n****************************************";
    cout << "\n   *** Sender-Only Test Request ***\n";
    cout << "\n Debug / Test /  master  /   sender   /   target   /   parameters\n";
    cout << endl << params << endl;
    cout << "\n****************************************\n\n";
}


/* -------------------------------------------------------------------
 *  getMultiOptions:  defaults are sent in as parameters & returned if 
 *     data is not entered.
 *
 * ------------------------------------------------------------------- */

void
DisplayFunc::getMultiOptions( string& addr, string& amount, string& port )
{
    string temp;
    cout << "\nIP Multicast Address in dotted decimal notation? [" << addr << "]  ";
    getFirstWord( temp );
    ( temp.empty() )? addr: addr=temp;

    cout << "Number of RTT tests? [" << amount << "]  ";
    getFirstWord( temp );
    ( temp.empty() )? amount: amount=temp;

    cout << "Port number? [" << port << "]  ";
    getFirstWord( temp );
    ( temp.empty() )? port: port=temp;
}


/* -------------------------------------------------------------------
 *  getPingOptions: defaults are sent in as parameters & returned if 
 *     data is not entered.
 *
 * ------------------------------------------------------------------- */

void
DisplayFunc::getPingOptions( string& amount, string& port )
{
    string temp;
    cout << "\nNumber of RTT tests? [" << amount << "]  ";
    getFirstWord( temp );
    ( temp.empty() )? amount: amount=temp;

    cout << "Port number? [" << port << "]  ";
    getFirstWord( temp );
    ( temp.empty() )? port: port=temp;
}


/* -------------------------------------------------------------------
 *  AskContinue( void )
 *
 * ------------------------------------------------------------------- */

int 
DisplayFunc::askContinue()
{
    cout << "\tContinue? [y]-yes []-no : ";
    return isContinue();
}


/* -----------------------------------------------------------------
 * prompt_for_Input: prompt_for_Input: Displays prompt and retrieves input.
 *
 * -----------------------------------------------------------------*/

string 
DisplayFunc::prompt_for_Input( const string prompt )
{
    string ans;
    cout << endl << prompt;
    cin >> ans;
    return ans;
}


/* -----------------------------------------------------------------
 *  AskQuestion
 *   First returns an answer
 *   Second Returns Continue 
 *
 * -----------------------------------------------------------------*/

void 
DisplayFunc::askQuestion( const string param, string& ans )
{
    cout << param;
    getline( cin, ans );
}
 
int 
DisplayFunc::askQuestion( const string param )
{
    cout << param;
    return isContinue();
}


/* -----------------------------------------------------------------
 *  getFirstWord: Returns first word of a string, return empty string if
 *     the 
 *
 * -----------------------------------------------------------------*/

void    
DisplayFunc::getFirstWord( string& ans )
{
    getAnswer(ans);
    string::size_type begIdx( ans.find_first_not_of(' ', 0) );
    if (begIdx==string::npos){
	ans = "";
	return;
    }
    string::size_type endIdx( ans.find_first_of(' ', begIdx) );
    (endIdx==string::npos)? endIdx=ans.length(): endIdx;
    ans.assign( ans, begIdx, (endIdx-begIdx) );
    return;
}


/* ------------------------------------------------------------------
 * askUserIperf: Appends a "/" token (delim) separator and iperf
 *               parameters to ctest.
 *
 * ------------------------------------------------------------------ */

void 
DisplayFunc::askUserIperf( string& ctest )
{
    displayAskUserIperf();
    askQuestion( "\nEnter Parameters: ", ctest );
    displayResult( "\nparameters are: ", ctest );
    if( ctest.empty() )
	ctest = EMPTY;
}


/* -------------------------------------------------------------------
 * askUserPing
 *
 * ------------------------------------------------------------------- */

void 
DisplayFunc::askUserPing( string& ctest )
{
    string amount("10"), port(UDPPING_PORT);

    getPingOptions( amount, port );
    ctest = "-n ";
    ctest.append( amount );
    ctest.append(" -p " );
    ctest.append( port );
}


/* -------------------------------------------------------------------
 * AskUserMulti
 *      dMultiAddr - global variable
 *
 * ------------------------------------------------------------------- */

int 
DisplayFunc::askUserMulti( string& ctest )
{
    string addr(MULTIPING_ADDR), amount("10"), port(UDPPING_PORT);

    getMultiOptions( addr, amount, port );
    unsigned long i = ntohl(inet_addr( addr.c_str() ));

    if( i < 0 ){
	cout << "ERROR: Multicast address\n";
	return -1;
    }

    if( !CLASSD_ADDR(i) ){
	cout << "ERROR: Multicast address Not ClassD\n";
	return -1;
    }

    ctest = "-m ";
    ctest.append( addr );
    ctest.append(" -n " );
    ctest.append( amount );
    ctest.append(" -p " );
    ctest.append( port );

    return 0;
}


/* -------------------------------------------------------------------
 * askUserTunedTcp : /udpping params/iperf params/bandwidth
 *
 * ------------------------------------------------------------------- */

void 
DisplayFunc::askUserTunedTcp( string& ctest )
{
    string params;

    askUserPing( ctest );
    askUserIperf( params );

    ctest.append( DELIM );
    ctest.append( params );
    ctest.append( DELIM );

    askQuestion( "\nEnter target bandwidth (Mbits/second): ", params );

    while( (params.empty()) || atof(params.c_str()) < 0 ){
        askQuestion( "\nERROR: Enter target bandwith (Mbits/second): ", params );
    }
    ctest.append( params );
}

/* -------------------------------------------------------------------
 * askUserTrace : 
 *
 * ------------------------------------------------------------------- */

void 
DisplayFunc::askUserTrace( string& ctest )
{
    askQuestion( "\nEnter Parameters: ", ctest );
    displayResult( "\nparameters are: ", ctest );
    if( ctest.empty() )
	ctest = EMPTY;
}
