//C++

#include <cstdlib>
#include <unistd.h>
#include <cstdio>
#include <memory.h>
#include <cerrno>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "openssl/ssl.h"
#include "openssl/crypto.h"
#include "openssl/err.h"

#define CHK_NULL(x) if ((x)==NULL) exit (1)
#define CHK_ERR(err,s) if ((err)==-1) { perror(s); exit(1); }
#define CHK_SSL(err) if ((err)==-1) { ERR_print_errors_fp(stderr); exit(2); }

#define CERTF "nettest.pem"
#define KEYF  "key.pem"

int main()
{
    int err;
    int listen_sd;
    int sd;
    long on = 1;
    struct sockaddr_in sa_serv;
    struct sockaddr_in sa_cli;
    size_t client_len;
    SSL_CTX* ctx;
    SSL*     ssl;
    int verify = SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT;
    X509*    client_cert;
    char*    str;
    char     buf [4096];
    SSL_METHOD *meth;
    SSL_load_error_strings();
    SSLeay_add_ssl_algorithms();
    meth = SSLv3_server_method();
    ctx = SSL_CTX_new (meth);
    if (!ctx) {
      ERR_print_errors_fp(stderr);
      exit(2);
    }

    if (SSL_CTX_use_certificate_file(ctx, CERTF, SSL_FILETYPE_PEM) <= 0) {
      ERR_print_errors_fp(stderr);
      exit(3);
    }
    if (SSL_CTX_use_PrivateKey_file(ctx, KEYF, SSL_FILETYPE_PEM) <= 0) {
      ERR_print_errors_fp(stderr);
      exit(4);
    }

    if (!SSL_CTX_check_private_key(ctx)) {
      fprintf(stderr,"Private key does not match the certificate public key\n");
      exit(5);
    }
    SSL_CTX_set_verify(ctx, verify, NULL);
    SSL_CTX_load_verify_locations (ctx, "CA.pem", NULL);
    SSL_CTX_set_client_CA_list (ctx, SSL_load_client_CA_file ("CA.pem"));
    listen_sd = socket (AF_INET, SOCK_STREAM, 0);   
    CHK_ERR(listen_sd, "socket");

    memset (&sa_serv, '\0', sizeof(sa_serv));
    sa_serv.sin_family      = AF_INET;
    sa_serv.sin_addr.s_addr = INADDR_ANY;
    sa_serv.sin_port        = htons (55556);          /* Server Port number */
    setsockopt(listen_sd, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof(on));
    err = bind(listen_sd, (struct sockaddr*) &sa_serv, sizeof (sa_serv)); 
    CHK_ERR(err, "bind");
    err = listen (listen_sd, 5);   
    CHK_ERR(err, "listen");
    client_len = sizeof(sa_cli);
    sd = accept (listen_sd, (struct sockaddr*) &sa_cli, &client_len);
    CHK_ERR(sd, "accept");
    close (listen_sd);

    printf ("Connection from %lx, port %x\n",
          sa_cli.sin_addr.s_addr, sa_cli.sin_port);
    ssl = SSL_new (ctx);                           
    CHK_NULL(ssl);
    SSL_set_fd (ssl, sd);
    err = SSL_accept (ssl);                        
    CHK_SSL(err);
    printf ("SSL connection using %s\n", SSL_get_cipher (ssl));
    client_cert = SSL_get_peer_certificate (ssl);
    if (client_cert != NULL) {
      printf ("Client certificate:\n");
      str = X509_NAME_oneline (X509_get_subject_name (client_cert), 0, 0);
      CHK_NULL(str);
      printf ("\t subject: %s\n", str);
      free (str);
      str = X509_NAME_oneline (X509_get_issuer_name  (client_cert), 0, 0);
      CHK_NULL(str);
      printf ("\t issuer: %s\n", str);
      free (str);
      X509_free (client_cert);
    } else
      printf ("Client does not have certificate.\n");
    close (sd);
    SSL_free (ssl);
    SSL_CTX_free (ctx);
}
